#####################################################################################################
# This file is part of FlagScreen.                                                                  #
#                                                                                                   #
# FlagScreen is free software: you can redistribute it and/or modify it under the terms of the      #
# GNU General Public License as published by the Free Software Foundation, either version 3 of      #
# the License, or (at your option) any later version.                                               #
#                                                                                                   #
# FlagScreen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without   #
# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU     #
# General Public License for more details.                                                          #
#                                                                                                   #
# You should have received a copy of the GNU General Public License along with FlagScreen.          #
# If not, see <http://www.gnu.org/licenses/>.                                                       #
#                                                                                                   #
# To download FlagScreen, visit                                                                     #
#   <http://www.ipb-halle.de/en/forschung/werkzeuge-und-datenbanken/flagscreen/>.                   #
#                                                                                                   #
# To cite FlagScreen in publications, use                                                           #
#   @article{Ranf22082011,                                                                          #
#       author  = { Ranf, Stefanie and Grimmer, Julia and Poeschl, Yvonne and Pecher, Pascal        #
#                   and Chinchilla, Delphine and Scheel, Dierk and Lee, Justin},                    #
#       title   = { Defense-Related Calcium Signaling Mutants Uncovered via a                       #
#                   Quantitative High-Throughput Screen in Arabidopsis thaliana},                   #
#       year    = { 2011},                                                                          #
#       doi     = { 10.1093/mp/ssr064},                                                             #
#       URL     = { http://mplant.oxfordjournals.org/content/early/2011/08/20/mp.ssr064.abstract},  #
#       journal = { Molecular Plant}                                                                #
#   }                                                                                               #
#                                                                                                   #
# The author of the file is Yvonne Poeschl (poeschl@informatik.uni-halle.de).                       #
#####################################################################################################

# The arguments of the easy.run() function as specified in the functions.r script:
############################################################################################################################
# easy run script performs normalization and plotting using the parameters specified below:                                #
#                                                                                                                          #
# filename:             name of the file containing the measurements of F and D                                            #
# quadrants:            number of quadrants for which the analysis should be done                                          #
#                       quadrants are numbered as follows:                                                                 #
#                           1:  A1  - H12                                                                                  #
#                           2:  I1  - P12                                                                                  #
#                           3:  A13 - H24                                                                                  #
#                           4:  I13 - P24                                                                                  #
# fnum:                 number of treatment measurements (F) per well                                                      #
# dnum:                 number of discharge measurements (D) per well                                                      #
# time.offset:          time offset used for normalization (in sec)                                                        #
# norm.by.calcium:      normalization can be done either by calcium concentration (in microM)                              #
#                       or by rate constant (k = L/Lmax) (TRUE or FALSE)                                                   #
# discharge.th:         discharge sum threshold for excluding wells from normalization                                     #
# bycol:                computation of the mean and the plotting can be done by col or by row (TRUE or FALSE)              #
# wt.filename:          name of the file containing the reference wild type measurements                                   #
#                       or NULL for not using wild type measurements from file                                             #
# wt.well.labels:       a vector of length equal to the number of quadrants                                                #
#                       each item containing the well to be used as wild type measurements                                 #
#                       it can be the empty string "" (no wells are used) or strings of the following format:              #
#                       "A1:B2:C3:A10" well labels of the quadrant to analyze concatenated with ":"                        #
#                       or NULL for not using wells                                                                        #
# save.wt:              save the mean wild type measurements or not (TRUE or FALSE)                                        #
# compute.plot.mean:    compute and plot mean of F (TRUE or FALSE)                                                         #
# quadrant.screen:      arrange the plots of the whole quadrant on one page (TRUE or FALSE)                                #
# ylim.max:             maximum value used for scaling the y-axis in the plots, if NULL ylim.max is automatically computed #
# plot.title:           a vector of length 12 (bycol = TRUE) or 8 (bycol = FALSE) containing the labels of the mutants     #
#                       or NULL for no additional plot title                                                               #
# plot.out:             directory to save the plots, "" is the working directory                                           #
# data.out:             directory to save the computed data, "" is the working directory                                   #
# out.base.name:        prefix of all output files                                                                         #
# save.f.d:             save processed treatment measurements, means and excluded candidates,                              #
#                       and discharge measurements (TRUE or FLASE)                                                         #
# verbose:              switching status report on or off (TRUE or FALSE)                                                  #
############################################################################################################################

################################
# SETTING R WORKING PARAMETERS #
################################
# setting the working directory
setwd("~/projects/FlagScreen/example");
# loading the method script
source("../functions.r");

########################################
# SETTING PARAMETERS FOR DATA HANDLING #
########################################
# The parameters to set with a short explanation. See the function description above 
# for a more detailed explanation and the valid parameter space.

# switching status report on or off (TRUE or FALSE)
my_verbose = TRUE;

# setting the directory to save the pdfs; make sure the the directory exists! (String)
my_plot.out = "plots/";

# setting the directory to save the data; make sure the the directory exists! (String)
my_data.out = "out/";

# setting prefix of all output files (String)
my_out.base.name = "out_example";

# save measurements or not (TRUE or FALSE)
my_save.f.d = TRUE;

# file containing the treatment and discharge measurements; refer to the README.txt for the format (String)
my_filename = "example_F_D.txt";

# file containing the wild type (reference) measurements; refer to the README.txt for the format (String or NULL)
#my_wt.filename = NULL;
my_wt.filename = "example_wt.txt";

# wild type measurements can also be obtained from the measurements be specifying the wells (String or NULL)
#my_wt.well.labels = c("A1:A10:H12:B2","I13:I22:P24:J14"); # if quadrants 1 and 4 are selected
#my_wt.well.labels = c("A1:A10:H12:B2","","","I13:I22:P24:J14"); # if quadrants 1, 2, 3, and 4 are selected
my_wt.well.labels = NULL;

# save wild type measurements or not (TRUE or FALSE)
my_save.wt = FALSE;


##################################
# SETTING MEASUREMENT PARAMETERS #
##################################
# number of treatment measurements (F) per well (number)
my_fnum = 100;
# number of discharge measurements (D) per well (number)
my_dnum = 10;
# time offset used for normalization (in sec) and scaling the x-axis in the plots (number)
my_time.offset = 6;

####################################
# SETTING PREPROCESSING PARAMETERS #
####################################
# if sum of discharge values is below this threshold then wells are excluded from normalization (number)
my_discharge.th = 0.5;

####################################
# SETTING NORMALIZATION PARAMETERS #
####################################
# normalization can be done either by calcium concentration (in microM) (TRUE) or by rate constant (k = L/Lmax) (FALSE)
my_norm.by.calcium = TRUE;

############################################
# SETTING ANALYSIS AND PLOTTING PARAMETERS #
############################################
# compute and plot the means or not (TRUE or FALSE)
my_compute.plot.mean = TRUE;

# the quadrants to analyze (vector)
#my_quadrants = c(1,4); # only quadrant 1 and 4 selected
my_quadrants = c(1,2,3,4); # all quadrants selected

# analysis and plotting can be done by column or by row (TRUE or FALSE)
my_bycol = TRUE;

# arrange the plots of the whole quadrant on one page (TRUE or FALSE)
my_quadrant.screen = TRUE;

# the y-axis is scaled automatically (NULL or number)
my_ylim.max = NULL;

# additional information for the title in the plots (NULL or vector)
#my_plot.title = paste(rep("m",12),1:12,sep=""); # additional plot titles for the 12 columns: m1 to m12
#my_plot.title = c("A","","","D","","","","","","","","L"); # 12 individual plot titles if bycol = TRUE or 8 individual plot titles if bycol = FALSE
my_plot.title = NULL;
			
###################################
# RUNNING THE easy.run() FUNCTION #
###################################
easy.run(	filename 			= my_filename,
			quadrants 			= my_quadrants,
			fnum 				= my_fnum,
			dnum 				= my_dnum,
			time.offset 		= my_time.offset,
			norm.by.calcium 	= my_norm.by.calcium,
			discharge.th 		= my_discharge.th,
			bycol 				= my_bycol,
			wt.filename 		= my_wt.filename,
			wt.well.labels 		= my_wt.well.labels,
			save.wt 			= my_save.wt,
			compute.plot.mean 	= my_compute.plot.mean,
			quadrant.screen		= my_quadrant.screen,
			ylim.max			= my_ylim.max,
			plot.title 			= my_plot.title,
			plot.out			= my_plot.out,
			data.out 			= my_data.out,
			out.base.name 		= my_out.base.name,
			save.f.d 			= my_save.f.d,
			verbose 			= my_verbose);

