#####################################################################################################
# This file is part of FlagScreen.                                                                  #
#                                                                                                   #
# FlagScreen is free software: you can redistribute it and/or modify it under the terms of the      #
# GNU General Public License as published by the Free Software Foundation, either version 3 of      #
# the License, or (at your option) any later version.                                               #
#                                                                                                   #
# FlagScreen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without   #
# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU     #
# General Public License for more details.                                                          #
#                                                                                                   #
# You should have received a copy of the GNU General Public License along with FlagScreen.          #
# If not, see <http://www.gnu.org/licenses/>.                                                       #
#                                                                                                   #
# To download FlagScreen, visit                                                                     #
#   <http://www.ipb-halle.de/en/forschung/werkzeuge-und-datenbanken/flagscreen/>.                   #
#                                                                                                   #
# To cite FlagScreen in publications, use                                                           #
#   @article{Ranf22082011,                                                                          #
#       author  = { Ranf, Stefanie and Grimmer, Julia and Poeschl, Yvonne and Pecher, Pascal        #
#                   and Chinchilla, Delphine and Scheel, Dierk and Lee, Justin},                    #
#       title   = { Defense-Related Calcium Signaling Mutants Uncovered via a                       #
#                   Quantitative High-Throughput Screen in Arabidopsis thaliana},                   #
#       year    = { 2011},                                                                          #
#       doi     = { 10.1093/mp/ssr064},                                                             #
#       URL     = { http://mplant.oxfordjournals.org/content/early/2011/08/20/mp.ssr064.abstract},  #
#       journal = { Molecular Plant}                                                                #
#   }                                                                                               #
#                                                                                                   #
# The author of the file is Yvonne Poeschl (poeschl@informatik.uni-halle.de).                       #
#####################################################################################################


############################################################################################################################
# easy run script performs normalization and plotting using the parameters specified below:                                #
#                                                                                                                          #
# filename:             name of the file containing the measurements of F and D                                            #
# quadrants:            number of quadrants for which the analysis should be done                                          #
#                       quadrants are numbered as follows:                                                                 #
#                           1:  A1  - H12                                                                                  #
#                           2:  I1  - P12                                                                                  #
#                           3:  A13 - H24                                                                                  #
#                           4:  I13 - P24                                                                                  #
# fnum:                 number of treatment measurements (F) per well                                                      #
# dnum:                 number of discharge measurements (D) per well                                                      #
# time.offset:          time offset used for normalization (in sec)                                                        #
# norm.by.calcium:      normalization can be done either by calcium concentration (in microM)                              #
#                       or by rate constant (k = L/Lmax) (TRUE or FALSE)                                                   #
# discharge.th:         discharge sum threshold for excluding wells from normalization                                     #
# bycol:                computation of the mean and the plotting can be done by col or by row (TRUE or FALSE)              #
# wt.filename:          name of the file containing the reference wild type measurements                                   #
#                       or NULL for not using wild type measurements from file                                             #
# wt.well.labels:       a vector of length equal to the number of quadrants                                                #
#                       each item containing the well to be used as wild type measurements                                 #
#                       it can be the empty string "" (no wells are used) or strings of the following format:              #
#                       "A1:B2:C3:A10" well labels of the quadrant to analyze concatenated with ":"                        #
#                       or NULL for not using wells                                                                        #
# save.wt:              save the mean wild type measurements or not (TRUE or FALSE)                                        #
# compute.plot.mean:    compute and plot mean of F (TRUE or FALSE)                                                         #
# quadrant.screen:      arrange the plots of the whole quadrant on one page (TRUE or FALSE)                                #
# ylim.max:             maximum value used for scaling the y-axis in the plots, if NULL ylim.max is automatically computed #
# plot.title:           a vector of length 12 (bycol = TRUE) or 8 (bycol = FALSE) containing the labels of the mutants     #
#                       or NULL for no additional plot title                                                               #
# plot.out:             directory to save the plots, "" is the working directory                                           #
# data.out:             directory to save the computed data, "" is the working directory                                   #
# out.base.name:        prefix of all output files                                                                         #
# save.f.d:             save processed treatment measurements, means and excluded candidates,                              #
#                       and discharge measurements (TRUE or FLASE)                                                         #
# verbose:              switching status report on or off (TRUE or FALSE)                                                  #
############################################################################################################################
easy.run <- function(   filename            = NULL,
                        quadrants           = c(1,2,3,4),
                        fnum                = 100,
                        dnum                = 10,
                        time.offset         = 1,
                        norm.by.calcium     = TRUE,
                        discharge.th        = 0.5,
                        bycol               = TRUE,
                        wt.filename         = NULL,
                        wt.well.labels      = NULL,
                        save.wt             = FALSE,
                        compute.plot.mean   = TRUE,
                        quadrant.screen     = FALSE,
                        ylim.max            = NULL,
                        plot.title          = NULL,
                        plot.out            = "",
                        data.out            = "",
                        out.base.name       = "out",
                        save.f.d            = TRUE,
                        verbose             = TRUE){
	
	if(is.null(filename)) stop("Please specify the name of the file containing the measurements!");
	
	if(sum(quadrants %in% 1:4) != length(quadrants)) stop("The quadrants you specified are not valid! The number of a quadrant can be 1, 2, 3, or 4.");
	
	# reads the wild type measurements
	if(is.null(wt.filename) & is.null(wt.well.labels)){
		if(verbose) print("No wild type data selected...");
		wt <- NULL;	
	}else if(!is.null(wt.filename) & !is.null(wt.well.labels)){
		stop("You set a path to a file containing wild type measurements AND coordinates on the well plate! Please set one of the options to NULL.")
	}else if(!is.null(wt.filename)){
		if(verbose) print("Reading wild type file...");
		wt <- read.wt(wt.filename);
	}else{
		if(length(quadrants) != length(wt.well.labels)) stop("Please set coordinates for each quadrant you selected!");
		if(verbose) print("Wild type from well plate...");
	}
	
	if(!is.null(plot.title)){
		if(bycol & length(plot.title) != 12) stop("The number of plot title differs from 12!");
		if(!bycol & length(plot.title) != 8) stop("The number of plot title differs from 8!");
	}
	
	
	if(verbose) print("Reading data file...");
	# reads data from file
	data <- read.file(filename);
		
	if(verbose) print(paste("Computations are done for the following quadrants: ", paste(quadrants, collapse = ", "), sep = ""));
	# runs the whole pipeline for each specified quadrant q
	for(q.id in 1:length(quadrants)){
		q <- quadrants[q.id];
		print("");
		print(paste("++++ Quadrant: ", q, " ++++", sep = ""));
		
		if(verbose) print("Splitting data into measurements F and discharge D...");
		# splits the data into measurements F and discharge D
		mats <- read.data(data = data, quadrant = q, fnum = fnum, dnum = dnum);
		# measurements: dim = 8 x 12 x time
		fmat <- mats$fmat;
		# discharge (mean values over time): dim = 8 x 12
		dmat <- mats$dmat;

		if(verbose) print("Normalizing measurements F...");
		# normalizes F with respect to D, time offset and the calcium content
		f.norm <- compute.norm(f = fmat, d = dmat, time.offset = time.offset, calcium = norm.by.calcium);

		if(verbose) print("Selecting discharge values below the discharge threshold...");
		# selects candidates with discharge values below the discharge threshold (discharge.th)
		exclude <- which(dmat < discharge.th, arr.ind = T);
		if(verbose){
			print(paste("The following measurements are excluded because their discharge was below ", discharge.th, ".", sep = ""));
			if(nrow(exclude) == 0){
				print("No candidates selected.");	
			}else{
				print(exclude);
			}
		}

		if(verbose){
			if(nrow(exclude) > 0) print("Excluding the selected candidates by setting their F value to NA...");
		}
		# excludes the selected candidates from mean computation and plotting by setting their values to NA
		if(nrow(exclude) > 0){
			f.norm.ex <- f.norm;
			for(i in 1:nrow(exclude)){
				f.norm.ex[exclude[i,1],exclude[i,2],] <- NA;
			}
		}else{
			f.norm.ex <- f.norm;	
		}

		
		# computes the means of the measurements F either by col (default) or by row
		if(compute.plot.mean){
			if(verbose) print("Computing means of the measurements F...");
			f.mean <- compute.mean(f = f.norm.ex, bycol = bycol);
		}else{
			f.mean <- NULL;	
		}
		
		if(is.null(wt.filename) & !is.null(wt.well.labels)){
			if(wt.well.labels[q.id] == ""){
				wt <- NULL;	
			}else{
				if(verbose) print("Computing means of the wild type measurements from well plate...");
				wt <- split.wt(wt.well.labels[q.id], q, f.norm.ex);
			}
		}

		# sets the label for the y-axis in the plot
		ylab <- expression(L/L[max]);
		if(norm.by.calcium) ylab <- expression(group("[",Ca^"2+","]")*~~group("(",symbol(m)*M,")"));
	
		if(verbose) print("Plotting measurements into pdf...");
		# plots the measurements profile for each well over time, the mean profile over time and the wild type profile over time
		plot.f(f = f.norm.ex, f.m = f.mean, wt = wt, ylim = ylim.max, bycol = bycol, dest = paste(plot.out, out.base.name, "_", q, ".pdf", sep = ""), time.offset = time.offset, ylab = ylab, plot.title = plot.title);
		
        if(quadrant.screen){
		    if(verbose) print("Plotting measurements into screen pdf...");
		    # plots the measurements profile for each well over time, the mean profile over time and the wild type profile over time
		    plot.f(f = f.norm.ex, f.m = f.mean, wt = wt, ylim = ylim.max, bycol = bycol, dest = paste(plot.out, out.base.name, "_", q, "_screen.pdf", sep = ""), time.offset = time.offset, ylab = ylab, plot.title = plot.title, quadrant.screen = TRUE);
        }
		if(save.f.d){
			if(verbose) print("Saving discharge values...");
			# saves the discharge values
			write.csv(dmat, paste(data.out, out.base.name, "_D_", q, ".txt", sep = ""), quote = FALSE);

			if(verbose) print("Saving excluded candidates...");
			# saves the excluded candidates
			write.csv(as.matrix(exclude[,2]), paste(data.out, out.base.name, "_ex_", discharge.th, "_", q, ".txt", sep = ""), quote = FALSE);

			if(verbose) print("Saving F values used for plotting...");
			# saves the measurement values F used for plotting
			write.csv(t(save.f(f.norm.ex)), paste(data.out, out.base.name, "_F_", q, ".txt", sep = ""), quote = FALSE);
		}
		
		# saves the mean F values
		if(compute.plot.mean & save.f.d){
			if(verbose) print("Saving mean F values...");
			write.csv(t(f.mean), paste(data.out, out.base.name, "_F.mean_", q, ".txt", sep = ""), quote = FALSE);
		}
		
		# saves the mean wt values
		if(save.wt & !is.null(wt)){
			if(verbose) print("Saving mean wt values...");
			write.table(t(wt), paste(data.out, out.base.name, "_wt_", q, ".txt", sep = ""), row.names = FALSE, col.names = FALSE, sep = ",", quote = FALSE);			
		}
	}
	
}

####################################################################
# reads the wild type measurements from a file (without header and #
# "\t" as separator) to vector                                     #
#                                                                  #
# fname:    name of the file containing the wild type measurements #
# return:   vector containing measurements                         #
####################################################################
read.wt <- function(fname){
	return(as.vector(as.matrix(read.table(file = fname, header = FALSE, stringsAsFactors = FALSE, sep= "," ))));
}

##########################################################################
# computes the wild type measurements from the specified wells           #
#                                                                        #
# well.labels:  a string containing the specified wells separated by ":" #
# quadrant:     the quadrant                                             #
# f:            measurements F                                           #
# return:       vector containing the mean wild type measurements        #
##########################################################################
split.wt <- function(well.labels, quadrant, f){
	wells <- get.quadrant.labels(quadrant);
	rnames <- wells$rnames;
	rpos <- 1:8;
	names(rpos) <- rnames;
	cnames <- wells$cnames;
	cpos <- 1:12;
	names(cpos) <- cnames;
	valid.wells <- wells$well.labels;
	
	well.labels <- strsplit(well.labels,":")[[1]];
	
	# checks if the specified wells are in the specified quadrant
	if(sum(well.labels %in% valid.wells) != length(well.labels)) stop(paste("One of the wells you select for the wild type is not in the selected quadrant ", quandrant, "!", sep = ""));
	
	# computes the coordinates of the specified wells
	M <- length(well.labels);
	well.pos <- matrix(NA, nrow = M, ncol = 2);
	for(n in 1:M){
			s <- strsplit(well.labels[n],"")[[1]];
			x <- rpos[s[1]];
			y <- cpos[paste(s[-1],collapse="")];
			well.pos[n,] <- c(x,y);
	}
	
	# computes the mean profile from the measurements of the specified wells
	reps <- dim(f)[3];
	wt <- rep(NA,reps);
	for(i in 1:reps){
		wt[i] <- mean(f[well.pos[,1],well.pos[,2],i], na.rm = TRUE);
	} 
	
	# returns the mean wild type measurements
	return(wt);
}

######################################################################
# returns the well labels of the specified quadrant q                #
#                                                                    #
# q:        the quadrant                                             #
# return:   a list of length 3 containing the well labels as vectors #
#           rnames:         row names, A to H or I to P              #
#           cnames:         col names, 1 to 12 or 13 to 24           #
#           well.labels:    all labels of the specified quadrant q   #
######################################################################
get.quadrant.labels <- function(q){
	if(q == 1){
		rnames <- LETTERS[1:8];
		cnames <- 1:12;
	}else if(q == 2){
		rnames <- LETTERS[9:16];
		cnames <- 1:12;
	}else if(q == 3){
		rnames <- LETTERS[1:8];
		cnames <- 13:24;
	}else if(q == 4){
		rnames <- LETTERS[9:16];
		cnames <- 13:24;
	}else{
		stop(paste("The quadrant ",q," you chose is not valid!"));	
	}
	return(list(rnames = rnames, cnames = cnames, well.labels = paste(rep(rnames, 12), rep(cnames, each = 8), sep = "")));
}

################################################################
# reads the measurements from a file to data.frame             #
# please read the README for information about the file format #
#                                                              #
# fname:    name of the file containing the measurements       #
# return:   data.frame containing measurements                 #
################################################################
read.file <- function(fname){
	data <- read.csv(fname, header = TRUE, stringsAsFactors = FALSE, row.names = 1);
	return(data);
}

###################################################################################
# splits the data contained in the data.frame into measurements F and discharge D #
#                                                                                 #
# data:     data.frame containing the measurements                                #
# quadrant: the quadrant the computation should be done                           #
# fnum:     number of measurements                                                #
# dnum:     number of measurements of the discharge                               #
# return:   list containing                                                       #
#           fmat: 3 dim array (8 x 12 x fnum) of measurements F                   #
#           dmat: 2 dim array (8 x 12) of summarized measurements D               #
###################################################################################
read.data <- function(data, quadrant, fnum, dnum){
	
	# well labels of the quadrant
	well.labels <- get.quadrant.labels(quadrant);
	rnames <- well.labels$rnames;
	cnames <- well.labels$cnames;
		
	# number of all measurements
	N <- nrow(data);
	if((N - fnum) != dnum) stop("Either fnum or dnum is not correctly set!");
	# initialization of fmat
	# the 3rd dimension contains the measurements over time
	fmat <- array(NA, dim = c(8, 12, fnum));
	rownames(fmat) <- rnames;
	colnames(fmat) <- cnames;
	
	# initialization of dmat
	# contains only the sum of measurement values over time
	dmat <- matrix(NA, nrow = 8, ncol = 12);
	rownames(dmat) <- rnames;
	colnames(dmat) <- cnames;
		
	# getting the correct column from data
	s <- 96 * (quadrant - 1) + 1;
	# copies the values from data to fmat and dmat
	for(j in 1:12){
		for(i in 1:8){
			fmat[i,j,] <- data[1:fnum,s];
			dmat[i,j] <- sum(data[(fnum + 1):N,s]);
			s <- s + 1;
		}
	}
	
	# setting 0s to NA
	idx <- which(fmat == 0, arr.ind = TRUE);
	if(nrow(idx) > 0) fmat[idx] <- NA;
	
	# returns fmat and dmat
	return(list(fmat = fmat, dmat = dmat));
}

####################################################################################
# normalizes the F values of one well over time                                    #
#                                                                                  #
# y:            F values of one well                                               #
# dis:          summarized D values of one well                                    #
# time.offset:  time offset                                                        #
# calcium:      if TRUE normalization is done by calcium concentration (in microM) #
#               instead of using the rate constant (k = L/Lmax)                    #
# return:       vector containing the normalized F values                          #
####################################################################################
compute.y <- function(y, dis, time.offset, calcium){
	# computes the cumulative sum from the right to the left
	denom <- rev(cumsum(rev(y)));
	
	# computes the rate constant
	k <- y / (time.offset * (denom + dis)); # rate constant
	if(calcium){
		y.new <- 10^(-(0.332588 * (-log(k, base=10)) + 5.5593)) * 1e6;
	}else{
		y.new <- k;	
	}
	
	# returns the normalized F values
	return(y.new)
}

####################################################################################
# normalizes the F values of the quadrant over time                                #
#                                                                                  #
# f:            F values                                                           #
# d:            summarized discharge values                                        #
# time.offset:  time offset                                                        #
# calcium:      if TRUE normalization is done by calcium concentration (in microM) #
#               instead of using the rate constant (k = L/Lmax)                    #
# return:       3 dim array (8 x 12 x fnum) of normalized measurements F           #
####################################################################################
compute.norm <- function(f, d, time.offset, calcium = TRUE){
	dim.f <- dim(f);
	nrow <- dim.f[1];
	ncol <- dim.f[2];
	m.num <- dim.f[3];
	
	# initialization of new 3 dim array for normalized F values
	f.norm <- array(dim=c(nrow, ncol, m.num));
	
	# normalizes the F values by going over each val
	for(i in 1:nrow){
		for(j in 1:ncol){
			f.norm[i,j,] <- compute.y(f[i,j,], d[i,j], time.offset, calcium);
		}	
	}	
	
	rownames(f.norm) <- rownames(f);
	colnames(f.norm) <- colnames(f);
	
	# returns the normalized F values
	return(f.norm)
}


######################################################################################
# computes the mean of the F values of each well                                     #
#                                                                                    #
# f:        F values                                                                 #
# bycol:    if TRUE the mean for each column over the values in the rows is computed #
# return:   matrix containing the mean F values                                      #
#           if bycol == TRUE    then dim = fnum x 12                                 #
#           if bycol == FALSE   then dim = fnum x 8                                  #
######################################################################################
compute.mean <- function(f, bycol = TRUE){
	f.dim <- dim(f);
	rows <- f.dim[1];
	cols <- f.dim[2];
	reps <- f.dim[3];
	
	rnames <- rownames(f);
	cnames <- colnames(f);
	
	num <- cols;
	# computes the column names depending on the bycol setting
	# if bycol == TRUE	then it is e.g. "A1 - H1"
	# if bycol == FLASE then it is e.g. "A1 - A12"
	colnames <- paste(rep(rnames[1], each=12), cnames, " - ", rep(rnames[8], each=12), cnames, sep = "");
	if(!bycol){
		num <- rows;	
		colnames <- paste(rnames, rep(cnames[1], each=8), " - ", rnames, rep(cnames[12], each=8), sep = "");
	}
	
	# initialization of 2 dim array for the mean F values
	# if bycol == TRUE	then dim = fnum x 12
	# if bycol == FALSE	then dim = fnum x 8
	f.mean <- matrix(nrow = reps, ncol = num);
	
	# computes the means
	for(j in 1:reps){
		if(bycol){
			f.mean[j,] <- colMeans(f[,,j], na.rm = TRUE);
		}else{
			f.mean[j,] <- rowMeans(f[,,j], na.rm = TRUE);
		}
	}
	
	colnames(f.mean) = colnames;
	rownames(f.mean) = 1:reps;
	
	# returns the mean F values
	return(f.mean)
}


###########################################################################################
# plots the curves into a pdf                                                             #
# the number of plots will be 8 + 1 (bycol = TRUE) or 12 + 1 (bycol = FALSE), the mean    #
# and the wild type are additionally plotted in an separate plot                          #
# for the quadrant screen the will be 96 + 12 or 96 + 8 plots                             #
#                                                                                         #
# f:                normalized and cleaned F values; col = "black"                        #
# f.m:              mean F values if NULL no means are plotted; col = "blue"              #
# wt:               wild type measurements if NULL no wild type is plotted; col = "red"   #
# ylim.max:         range (0,ylim.max) the plots will be scaled to                        #
#                   if NULL the maximum value of all measurements to plot is computed     #
# bycol:            plotting by col or row as in computing the means of the F values      #
# dest:             directory the pdf should be saved if NULL a x11 window will be opened #
# time.offset:      time offset is used to scale the x-axis                               #
# ylab:             label of the y-axis                                                   #
# plot.title:       additional information that can be used in the title                  #
#                   if NULL no additional information is used                             #
# quadrant.screen:  if TRUE all plots of the quadrant will be arranged on one pdf page    #
###########################################################################################
plot.f <- function(f, f.m, wt, ylim.max = NULL, bycol = TRUE, dest = NULL, time.offset, ylab = "", plot.title = NULL, quadrant.screen = FALSE){
	# number of plots on one pdf page, either 8 + 1 or 12 + 1
	s <- c(3,3);
	if(!bycol) s <- c(3,5);
	
	# number of plots on one pdf page, either 96 + 12 or 96 + 8
	if(quadrant.screen){
		s <- c(9,12);
		if(!bycol) s <- c(8,13);
	}
	
	
	# transposes f if needed
	if(!bycol){
		rownames.t <- colnames(f);
		colnames.t <- rownames(f);
		dim.f.t <- dim(f);
		nrow.t <- dim.f.t[1];
		ncol.t <- dim.f.t[2];
		reps.t <- dim.f.t[3];
		f.t <- array(dim = c(ncol.t, nrow.t, reps.t));
		for(m in 1:reps.t){
			f.t[,,m] <- t(f[,,m]);
		}
		f <- f.t;

		rownames(f) <- rownames.t;
		colnames(f) <- colnames.t;
	}
	
	# checks the number of plots
	if((s[1]*s[2]) < nrow(f)){
		stop("Not enough screens to plot in! Please resize s!");
	}
	
	dim.f <- dim(f);
	rows <- dim.f[1];
	reps <- dim.f[3];
	cols <- dim.f[2];
	reps.wt <- max(reps, length(wt));
	
	xlim.max <- reps.wt;
	xlim <- c(0, xlim.max);
	
	# computes the maximum value of all values to plot
	if(is.null(ylim.max)){
		if(is.null(wt) & is.null(f.m)){
			ylim.max <- max(range(f, na.rm = TRUE));
		}else if(is.null(wt)){
			ylim.max <- max(c(range(f, na.rm = TRUE), range(f.m, na.rm = TRUE)));
		}else if(is.null(f.m)){
			ylim.max <- max(c(range(f, na.rm = TRUE), range(wt, na.rm = TRUE)));
		}else{
			ylim.max <- max(c(range(f, na.rm = TRUE), range(wt, na.rm = TRUE), range(f.m, na.rm = TRUE)));
		}
	}
	ylim <- c(0, ylim.max);

	# plot mean?
	plot.m <- TRUE;
	if(is.null(f.m)){
		plot.m <- FALSE;	
	}
	
	# plot wild type?
	plot.wt <- TRUE;
	if(is.null(wt)){
		plot.wt <- FALSE;	
	}
	
	# parameters for plotting
	xlab <- "time (min)";
	
	cex.lab <- 1;
	cex.main <- 1.2;
	lwd <- 2;
	
	# if no destination is specified a graphic device is opened
	if(is.null(dest)){
		x11();
	}
	else{
		pdf(file = dest, onefile = TRUE, width = s[2] * 3, height = s[1] * 3);	}
	
	# sets parameters
	if(bycol & quadrant.screen){
		par(mfcol = s, cex.lab = cex.lab, cex.main = cex.main);
	}else{
		par(mfrow = s, cex.lab = cex.lab, cex.main = cex.main);
	}
	
	# the legend for the additional plot
	legend <- c("measurement");
	col <- c("black");
	if(plot.m){
		legend <- c(legend, "mean");
		col <- c(col, "blue");
	}
	if(plot.wt){
		legend <- c(legend, "wild type");
		col <- c(col, "red");
	}
	
	# plotting
	for(i in 1:cols){
		# for each well its curve of the F values (black), the mean values (blue) and the wild type values (red) are plotted
		for(j in 1:rows){
			main <- paste(rownames(f)[j], colnames(f)[i], sep = "");
			if(!bycol) main <- paste(colnames(f)[i], rownames(f)[j], sep = "");
			if(!is.null(plot.title)) main <- paste(main, plot.title[i], sep = " - ");
			plot(x = 1:reps, y = f[j,i,], type = 'l', main = main, xlab = xlab, ylab = ylab, xlim = xlim, ylim = ylim, lwd = lwd, axes = FALSE);
			axis(2);
			axis(1, at = seq(from = 0, to = xlim.max, by = 20), labels = seq(from = 0, to = xlim.max, by = 20) * time.offset / 60 );
			if(plot.m) lines(x = 1:reps, f.m[,i], col = "blue", lwd = lwd);
			if(plot.wt) lines(x = 1:reps.wt, wt, col = "red", lwd = lwd);
		}
		
		# plots only mean and wild type values in a separate plot
		plot(0, 0, col = "white", xlab = xlab, ylab = ylab, xlim = xlim, ylim = ylim, main = paste(colnames(f.m)[i],plot.title[i],sep = " "), lwd = lwd, axes = FALSE);
		axis(2);
		axis(1, at = seq(from = 0, to = xlim.max, by = 20), labels = seq(from = 0, to = xlim.max, by = 20) * time.offset / 60 );
		if(plot.m) lines(x = 1:reps, f.m[,i], col = "blue", lwd = lwd);
		if(plot.wt) lines(x = 1:reps.wt, wt, col = "red", lwd = lwd);
		legend("topright", legend = legend, col = col, lty = 1, lwd = lwd, box.col = "white");
		
		# creates empty plots to fill the pdf page
		if(!quadrant.screen){
			if((rows+2) < (s[1] * s[2])){
				for(j in (rows + 2):(s[1] * s[2])){
					plot(0, 0, col = "white", axes = FALSE, xlab = "", ylab = "");
				}	
			}
		}
	}
	
	# closes the device
	if(!is.null(dest)) dev.off();
}

#######################################################
# converts the 3 dim F value array into a 2 dim array #
#                                                     #
# f:        3 dim F value array                       #
# return:   2 dim F value array                       #
#######################################################
save.f <- function(f){
	
	dim.f <- dim(f);
	rows <- dim.f[1];
	cols <- dim.f[2];
	reps <- dim.f[3];
	
	colnames <- colnames(f);
	rownames <- rownames(f);
	
	flat.f <- matrix(NA, nrow = reps, ncol = (rows * cols));
	flat.colnames <- vector(mode = "character", length = (rows * cols));
	flat.rownames <- 1:reps;
	
	for(i in 1:rows){
		for(j in 1:cols){
			flat.f[, (i-1) * cols + j] <- f[i,j,];
			flat.colnames[(i-1) * cols + j] <- paste(rownames[i], colnames[j], sep = "");
		}
	}
	
	rownames(flat.f) <- flat.rownames;
	colnames(flat.f) <- flat.colnames;
	
	# returns the 2 dim array
	return(flat.f)
}